package clogs.parser;

import java_cup.runtime.SymbolFactory;

%%

%class ClogsLexer
%public
%{
   private SymbolFactory sf;
   public ClogsLexer (java.io.InputStream r, SymbolFactory sf)
   {
     this (r);
     this.sf = sf;
   }
%}
%eofval{
  return sf.newSymbol ("EOF", sym.EOF);
%eofval}

%unicode

%cup
%cupdebug

%char
%column
%line


ALPHA=[A-Za-z_]
DIGIT=[0-9]
NONNEWLINE_WHITE_SPACE_CHAR=[\ \t\b\012]
NEWLINE=\r|\n|\r\n
WHITE_SPACE_CHAR=[\n\r\ \t\b\012]
CHAR_TEXT=\\\"|[^\n\r\"]
STRING_TEXT=(\\\"|[^\n\r\"]|\\{WHITE_SPACE_CHAR}+\\)*
COMMENT_TEXT=([^*/\n]|[^*\n]"/"[^*\n]|[^/\n]"*"[^/\n]|"*"[^/\n]|"/"[^*\n])*
IDENT={ALPHA}({ALPHA}|{DIGIT}|_)*

%% 

<YYINITIAL> {
  "if" { return sf.newSymbol ("if", sym.IF); }
  "else" { return sf.newSymbol ("else", sym.ELSE); }
  "while" { return sf.newSymbol ("while", sym.WHILE); }
  "goto" { return sf.newSymbol ("goto", sym.GOTO); }
  "return" { return sf.newSymbol ("return", sym.RETURN); }
  "skip" { return sf.newSymbol ("return", sym.SKIP); }
  "new" { return sf.newSymbol ("new", sym.NEW); }
  "void" { return sf.newSymbol ("void", sym.VOID); }
  "int" { return sf.newSymbol ("int", sym.INT); }

  "[" { return sf.newSymbol ("LeftBrack", sym.LBRACK); }
  "]" { return sf.newSymbol ("RightBrack", sym.RBRACK); }
  "{" { return sf.newSymbol ("LeftBrace", sym.LBRACE); }
  "}" { return sf.newSymbol ("RightBrace", sym.RBRACE); }
  "(" { return sf.newSymbol ("LeftParen", sym.LPAREN); }
  ")" { return sf.newSymbol ("RightParen", sym.RPAREN); }
  "," { return sf.newSymbol ("Comma", sym.COMMA); }
  ";" { return sf.newSymbol ("Semicolon", sym.SEMI); }
  ":" { return sf.newSymbol ("Colon", sym.COLON); }
  "::" { return sf.newSymbol ("ColonColon", sym.COLONCOLON); }
  "=" { return sf.newSymbol ("Equals", sym.EQUALS); }
  "!" { return sf.newSymbol ("Bang", sym.BANG); }
  "==" { return sf.newSymbol ("EqualsEquals", sym.EQUALSEQUALS); }
  "!=" { return sf.newSymbol ("BangEquals", sym.BANGEQUALS); }
  "<" { return sf.newSymbol ("LessThan", sym.LESSTHAN); }
  ">" { return sf.newSymbol ("GreaterThan", sym.GREATERTHAN); }
  "<=" { return sf.newSymbol ("Leq", sym.LEQ); }
  ">=" { return sf.newSymbol ("Geq", sym.GEQ); }
  "|" { return sf.newSymbol ("Bar", sym.BAR); }
  "^" { return sf.newSymbol ("Caret", sym.CARET); }
  "&" { return sf.newSymbol ("Ampersand", sym.AMP); }
  "||" { return sf.newSymbol ("BarBar", sym.BARBAR); }
  "&&" { return sf.newSymbol ("AmpersandAmpersand", sym.AMPAMP); }
  "+" { return sf.newSymbol ("Plus", sym.PLUS); }
  "-" { return sf.newSymbol ("Minus", sym.MINUS); }
  "*" { return sf.newSymbol ("Times", sym.TIMES); }
  "/" { return sf.newSymbol ("Divide", sym.DIVIDE); }
  "%" { return sf.newSymbol ("Mod", sym.MOD); }

  {NONNEWLINE_WHITE_SPACE_CHAR}+ { }

  {IDENT} {
    return sf.newSymbol ("Identifier", sym.IDENTIFIER, yytext ());
  }

  {DIGIT}+ {
    int i = Integer.parseInt (yytext ());
    return sf.newSymbol ("IntegerConstant", sym.INTEGER_CONSTANT, new Integer (i));
  }

  ' {CHAR_TEXT} ' {
    return sf.newSymbol ("CharacterConstant", sym.CHARACTER_CONSTANT, yytext ().charAt (1));
  }

  \" {STRING_TEXT} \" {
    String str = yytext ().substring (1, yylength () - 1);
    return sf.newSymbol ("StringConstant", sym.STRING_CONSTANT, str);
  }
}

{NEWLINE} { }

. {
  System.out.println ("Illegal character: <" + yytext () + ">");
  Utility.error (Utility.E_UNMATCHED);
}

