package clogs.transform;

import clogs.ast.*;
import clogs.util.List;
import clogs.util.Optional;

public class EliminateHighLevelControl extends Phase
{
  public List<ExtDecl> transform (List<ExtDecl> edecls)
  {
    List<ExtDecl> result = List.nil ();

    for (ExtDecl edecl : edecls) {
      if (edecl instanceof Decl) {
        result = result.snoc (edecl);

      } else if (edecl instanceof FunDef) {
        FunDef fundef = (FunDef) edecl;
        result = result.snoc (new FunDef (fundef.type, fundef.name, fundef.params, transformBody (fundef.body)));

      } else {
        throw new RuntimeException ("Missing ExtDecl case: " + edecl.getClass ().getName ());
      }
    }

    return result;
  }


  static StatCompound transformBody (StatCompound statC) {
    List<Stat> statsNew = List.nil ();
    for (Stat stat : statC.stats) {
      statsNew = statsNew.concat (transformStat (stat));
    }
    return new StatCompound (statC.decls, statsNew);
  }


  static List<Stat> transformStat (Stat stat) 
  {
    List<Stat> result = List.nil ();

    if (stat instanceof StatCompound) {
      StatCompound statC = (StatCompound) stat;
      result = result.snoc (transformBody (statC).addLabels (statC.labels));

    } else if (stat instanceof StatExp) {
      result = result.snoc (stat);

    } else if (stat instanceof StatGoto) {
      result = result.snoc (stat);

    } else if (stat instanceof StatIf) {
      //     if (exp) statT else statF
      // --> 
      //     if (exp) 
      //       goto l1; 
      //     else
      //       goto l2;
      //   l1:
      //     skip;
      //     statT
      //     goto l3;
      //   l2:
      //     skip;
      //     statF
      //     goto l3;
      //   l3:
      //     skip
      StatIf statI = (StatIf) stat;
      String l1 = Fresh.getLabel ("true");
      String l2 = Fresh.getLabel ("false");
      String l3 = Fresh.getLabel ("end_if");
      result = result.snoc (new StatIf (statI.exp, new StatGoto (l1), new StatGoto (l2)).addLabels (statI.labels));
      result = result.snoc (new StatSkip ().addLabel (l1));
      result = result.concat (transformStat (statI.statT));
      result = result.snoc (new StatGoto (l3));
      result = result.snoc (new StatSkip ().addLabel (l2));
      result = result.concat (transformStat (statI.statF));
      result = result.snoc (new StatGoto (l3));
      result = result.snoc (new StatSkip ().addLabel (l3));

    } else if (stat instanceof StatReturn) {
      result = result.snoc (stat);

    } else if (stat instanceof StatSkip) {
      result = result.snoc (stat);

    } else if (stat instanceof StatWhile) {
      //     while (exp) stat
      // --> 
      //     goto l2;
      //   l1:
      //     skip;
      //     stat
      //   l2:
      //     if (exp) 
      //       goto l1 
      //     else 
      //       goto l3;
      //   l3:
      //     skip;
      StatWhile statW = (StatWhile) stat;
      String l1 = Fresh.getLabel ("body_while");
      String l2 = Fresh.getLabel ("test_while");
      String l3 = Fresh.getLabel ("end_while");
      result = result.snoc (new StatGoto (l2).addLabels (statW.labels));
      result = result.snoc (new StatSkip ().addLabel (l1));
      result = result.concat (transformStat (statW.stat));
      result = result.snoc (new StatIf (statW.exp, new StatGoto (l1), new StatGoto (l3)).addLabel (l2));
      result = result.snoc (new StatSkip ().addLabel (l3));

    } else {
      throw new RuntimeException ("Missing Stat case: " + stat.getClass ().getName ());
    }

    return result;
  }
}

