package clogs.transform;

import clogs.ast.*;
import clogs.util.List;
import clogs.util.Optional;


public class EliminateInitializers extends Phase
{
  public List<ExtDecl> transform (List<ExtDecl> edecls)
  {
    List<ExtDecl> result = List.nil ();

    for (ExtDecl edecl : edecls) {
      if (edecl instanceof Decl) {
        result = result.snoc (edecl);

      } else if (edecl instanceof FunDef) {
        FunDef fundef = (FunDef) edecl;
        result = result.snoc (new FunDef (fundef.type, fundef.name, fundef.params, transformBody (fundef.body)));

      } else {
        throw new RuntimeException ("Missing ExtDecl case: " + edecl.getClass ().getName ());
      }
    }

    return result;
  }


  StatCompound transformBody (StatCompound statC) {
    List<Decl> declsNew = List.nil ();
    List<Stat> statsNew = List.nil ();

    for (Decl decl : statC.decls) {
      if (decl.eo.isEmpty ()) {
        declsNew = declsNew.snoc (decl);
      } else {
        declsNew = declsNew.snoc (new Decl (decl.type, decl.name, new Optional<Exp> ()));
        statsNew = statsNew.snoc (new StatExp (new ExpAssign (new ExpVar (decl.name), decl.eo.get ())));
      }
    }

    for (Stat stat : statC.stats) {
      statsNew = statsNew.snoc (transformStat (stat));
    }

    return new StatCompound (declsNew, statsNew);
  }


  Stat transformStat (Stat stat) 
  {
    if (stat instanceof StatCompound) {
      StatCompound statC = (StatCompound) stat;
      return transformBody (statC).addLabels (statC.labels);

    } else if (stat instanceof StatExp) {
      return stat;

    } else if (stat instanceof StatGoto) {
      return stat;

    } else if (stat instanceof StatIf) {
      StatIf statI = (StatIf) stat;
      return new StatIf (statI.exp, transformStat (statI.statT), transformStat (statI.statF)).addLabels (statI.labels);

    } else if (stat instanceof StatReturn) {
      return stat;

    } else if (stat instanceof StatSkip) {
      return stat;

    } else if (stat instanceof StatWhile) {
      StatWhile statW = (StatWhile) stat;
      return new StatWhile (statW.exp, transformStat (statW.stat)).addLabels (statW.labels);

    } else {
      throw new RuntimeException ("Missing Stat case: " + stat.getClass ().getName ());
    }
  }
}

