package clogs.transform;

import clogs.ast.*;
import clogs.staticanalysis.GatherLabelsInStat;
import clogs.util.List;
import clogs.util.Map;
import clogs.util.Optional;


public class RenameLabelsUniquely extends Phase
{
  public List<ExtDecl> transform (List<ExtDecl> edecls)
  {
    List<ExtDecl> result = List.nil ();

    for (ExtDecl edecl : edecls) {
      if (edecl instanceof Decl) {
        result = result.snoc (edecl);

      } else if (edecl instanceof FunDef) {
        FunDef fundef = (FunDef) edecl;
        result = result.snoc (new FunDef (fundef.type, fundef.name, fundef.params, transformBody (fundef.body)));

      } else {
        throw new RuntimeException ("Missing ExtDecl case: " + edecl.getClass ().getName ());
      }
    }

    return result;
  }


  StatCompound transformBody (StatCompound statC) {
    GatherLabelsInStat glisc = new GatherLabelsInStat ();
    glisc.gather (statC);
    Map<String, String> labelRenaming = Map.empty ();
    for (String oldLabel : glisc.labelsDefined) {
      String newLabel = Fresh.getLabel (oldLabel);
      labelRenaming = labelRenaming.add (oldLabel, newLabel);
    }

    List<Stat> statsNew = List.nil ();
    for (Stat stat : statC.stats) {
      statsNew = statsNew.snoc (transformStat (labelRenaming, stat));
    }

    return new StatCompound (statC.decls, statsNew);
  }


  Stat transformStat (Map<String, String> labelRenaming, Stat stat) 
  {
    List<String> newLabels = List.nil ();
    for (String oldLabel : stat.labels) {
      newLabels = newLabels.snoc (labelRenaming.get (oldLabel));
    }

    if (stat instanceof StatCompound) {
      StatCompound statC = (StatCompound) stat;
      List<Stat> statsNew = List.nil ();
      for (Stat s : statC.stats) {
        statsNew = statsNew.snoc (transformStat (labelRenaming, s));
      }
      return new StatCompound (statC.decls, statsNew).removeLabels ().addLabels (newLabels); 

    } else if (stat instanceof StatExp) {
      return stat.removeLabels ().addLabels (newLabels);

    } else if (stat instanceof StatGoto) {
      StatGoto statG = (StatGoto) stat;
      return new StatGoto (labelRenaming.get (statG.target)).removeLabels ().addLabels (newLabels);

    } else if (stat instanceof StatIf) {
      StatIf statI = (StatIf) stat;
      return new StatIf (statI.exp, transformStat (labelRenaming, statI.statT), transformStat (labelRenaming, statI.statF)).removeLabels ().addLabels (newLabels);

    } else if (stat instanceof StatReturn) {
      return stat.removeLabels ().addLabels (newLabels);

    } else if (stat instanceof StatSkip) {
      return stat.removeLabels ().addLabels (newLabels);

    } else if (stat instanceof StatWhile) {
      StatWhile statW = (StatWhile) stat;
      return new StatWhile (statW.exp, transformStat (labelRenaming, statW.stat)).removeLabels ().addLabels (newLabels);

    } else {
      throw new RuntimeException ("Missing Stat case: " + stat.getClass ().getName ());
    }
  }
}

