import java_cup.runtime.SymbolFactory;
import java_cup.runtime.ComplexSymbolFactory;

%%

%class SkeletonLexer
%public
%{
   private SymbolFactory sf = new ComplexSymbolFactory ();
   public SkeletonLexer (java.io.InputStream r, SymbolFactory sf)
   {
     this (r);
     this.sf = sf;
   }
%}
%eofval{
  return sf.newSymbol ("EOF", sym.EOF);
%eofval}

%unicode
%cup
%cupdebug
%char
%column
%line

ALPHA=[A-Za-z_]
DIGIT=[0-9]
NONNEWLINE_WHITE_SPACE_CHAR=[\ \t\b\012]
NEWLINE=\r|\n|\r\n
IDENT={ALPHA}({ALPHA}|{DIGIT}|_)*

%states A
%xstates B

%% 

<YYINITIAL,A> "(" { System.out.println("State B"); yybegin(B); } 
<B> ")" { System.out.println("State A"); yybegin(A); } 
<YYINITIAL,A,B> { 
  {IDENT} { return sf.newSymbol ("Identifier", sym.IDENTIFIER, yytext ()); }
  {DIGIT}+ {
    int i = Integer.parseInt (yytext ());
    return sf.newSymbol ("IntegerConstant", sym.INTEGER_CONSTANT, new Integer (i));
  }
}

{NONNEWLINE_WHITE_SPACE_CHAR}+ { }

{NEWLINE} { }

<YYINITIAL,A,B> . {
  System.out.println ("Illegal character: <" + yytext () + ">");
}

/*
The difference between state and xstate shows up when looking at
REs that are not assigned to any state.  A state will include such
patterns, but an xstate will not.  In the example, I have two REs that
are not assigned any state.  These three only match in states
YYINITIAL and A, but not in B which is an xstate.

The file will parse

  dog cat (pig)

but not

  dog (cat pig)

since spaces are not allowed in state B.  (The parser works in state B
between parens, and in state YYINITIAL or A otherwise).
*/