abstract class Exp
{
}


class ExpApply extends Exp
{
  final Exp operator;
  final Exp operand;

  ExpApply (Exp operator, Exp operand)
  {
    this.operator = operator;
    this.operand = operand;
  }

  public String toString ()
  {
    return "(" + operator + " " + operand + ")";
  }
}


class ExpCons extends Exp
{
  final Exp head;
  final Exp tail;

  ExpCons (Exp head, Exp tail)
  {
    this.head = head;
    this.tail = tail;
  }

  public String toString ()
  {
    return "(" + head + "::" + tail + ")";
  }
}


class ExpFun extends Exp
{
  final String functionName; 
  final String parameterName; 
  final Exp body; 

  ExpFun (String functionName, String parameterName, Exp body)
  {
    this.functionName = functionName;
    this.parameterName = parameterName;
    this.body = body;
  }

  public String toString ()
  {
    return "(fun " + functionName + " " + parameterName + " => " + body + ")";
  }
}


class ExpIfThenElse extends Exp
{
  final Exp condition;
  final Exp trueBranch;
  final Exp falseBranch;

  ExpIfThenElse (Exp condition, Exp trueBranch, Exp falseBranch)
  {
    this.condition = condition;
    this.trueBranch = trueBranch;
    this.falseBranch = falseBranch;
  }

  public String toString ()
  {
    return "(if " + condition + " then " + trueBranch + " else " + falseBranch + ")";
  }
}


class ExpInt extends Exp
{
  final int value;

  ExpInt (int value)
  {
    this.value = value;
  }

  public String toString ()
  {
    return Integer.toString (value);
  }
}


class ExpLet extends Exp
{
  final String varName;
  final Exp exp1;
  final Exp exp2;

  ExpLet (String varName, Exp exp1, Exp exp2)
  {
    this.varName = varName;
    this.exp1 = exp1;
    this.exp2 = exp2;
  }

  public String toString ()
  {
    return "(let " + varName + " = " + exp1 + " in " + exp2 + ")";
  }
}


class ExpNil extends Exp
{
  ExpNil ()
  {
  }

  public String toString ()
  {
    return "[]";
  }
}


class ExpString extends Exp
{
  final String value;

  ExpString (String value)
  {
    this.value = value;
  }

  public String toString ()
  {
    return "\"" + value + "\"";
  }
}


class ExpVar extends Exp
{
  final String name;

  ExpVar (String name)
  {
    this.name = name;
  }

  public String toString ()
  {
    return name;
  }
}
