import util.*;


class Infer
{
  Constraints constraints = new Constraints ();
  Map typeSchemes; // Maps from variable names (String instances) to TypeScheme instances.


  Infer (Map typeSchemes)
  {
    this.typeSchemes = typeSchemes;
  }


  // Context maps from variable names (String instances) to Type instances.
  Type inferType (Map context, Exp e)
    throws TypeCheckerException
  {
    Type result;

    if (e instanceof ExpApply) {
      ExpApply expA = (ExpApply) e;
      Type t1 = inferType (context, expA.operator);
      Type t2 = inferType (context, expA.operand);
      Type A = new TypeVar (Fresh.getFreshTypeVariable ());
      Type B = new TypeVar (Fresh.getFreshTypeVariable ());
      constraints.add (t1, new TypeFun (A, B));
      constraints.add (t2, A);
      result = B;

    } else if (e instanceof ExpCons) {
      ExpCons expC = (ExpCons) e;
      Type t1 = inferType (context, expC.head);
      Type t2 = inferType (context, expC.tail);
      constraints.add (new TypeList (t1), t2);
      result = t2;

    } else if (e instanceof ExpFun) {
      ExpFun expF = (ExpFun) e;
      Type A = new TypeVar (Fresh.getFreshTypeVariable ());
      Type B = new TypeVar (Fresh.getFreshTypeVariable ());
      TypeFun t1 = new TypeFun (A, B);
      context = context.add (expF.functionName, t1);
      context = context.add (expF.parameterName, A);
      Type t2 = inferType (context, expF.body);
      constraints.add (t2, B);
      result = t1;

    } else if (e instanceof ExpIfThenElse) {
      ExpIfThenElse expI = (ExpIfThenElse) e;
      Type t1 = inferType (context, expI.condition);
      Type t2 = inferType (context, expI.trueBranch);
      Type t3 = inferType (context, expI.falseBranch);
      constraints.add (t1, new TypeBase ("boolean"));
      constraints.add (t2, t3);
      result = t2;

    } else if (e instanceof ExpInt) {
      result = new TypeBase ("int");

    } else if (e instanceof ExpLet) {
      throw new RuntimeException ("Not handling ExpLet");

    } else if (e instanceof ExpNil) {
      result = new TypeList (new TypeVar (Fresh.getFreshTypeVariable ()));

    } else if (e instanceof ExpString) {
      result = new TypeBase ("string");

    } else if (e instanceof ExpVar) {
      ExpVar expV = (ExpVar) e;
      Type type = (Type) context.get (expV.name);
      if (type == null) {
        TypeScheme typeScheme = (TypeScheme) typeSchemes.get (expV.name);
        if (typeScheme == null) {
          throw new TypeCheckerException ("Unbound variable: " + e);
        }
        type = typeScheme.instantiateWithFreshTypeVariables ();
      }
      result = type;
      
    } else {
      throw new RuntimeException ("Missing case for: " + e);
    }

    System.out.println ("Inferred " + e + " : " + result);

    return result;
  }
}
