import util.*;


abstract class Type
{
  // subst maps from type variable names (String instances) to Type instances
  abstract Type substitute (Map subst);
}


class TypeBase extends Type
{
  final String name;

  TypeBase (String name) 
  {
    this.name = name;
  }

  public String toString ()
  {
    return name;
  }

  Type substitute (Map subst)
  {
    return this;
  }
}


class TypeFun extends Type
{
  final Type type1;
  final Type type2;

  TypeFun (Type type1, Type type2) 
  {
    this.type1 = type1;
    this.type2 = type2;
  }

  public String toString ()
  {
    return "(" + type1 + " -> " + type2 + ")";
  }

  Type substitute (Map subst)
  {
    return new TypeFun (type1.substitute (subst), type2.substitute (subst));
  }
}


class TypeList extends Type
{
  final Type type;

  TypeList (Type type) 
  {
    this.type = type;
  }

  public String toString ()
  {
    return "(" + type + " list)";
  }

  Type substitute (Map subst)
  {
    return new TypeList (type.substitute (subst));
  }
}


class TypeVar extends Type
{
  final String name;

  TypeVar (String name) 
  {
    this.name = name;
  }

  public String toString ()
  {
    return "'" + name;
  }

  Type substitute (Map subst)
  {
    Type replacement = (Type) subst.get (name);
    if (replacement == null) {
      return this;
    } else {
      return replacement;
    }
  }
}
