package util;

import java.util.HashSet;


public abstract class List
{
  final static List nil = new ListNil ();
  

  public static List nil ()
  {
    return nil;
  }


  public static List singleton (Object o)
  {
    return nil ().cons (o);
  }


  public List cons (Object head)
  {
    return new ListCons (head, this);
  }


  public List snoc (Object last)
  {
    return this.concat (singleton (last));
  }


  public Object get (int index) 
  {
    int i = index;
    List l = this;
    while ((i > 0) && (!l.isNil ())) {
      i--;
      l = l.tail ();
    }

    if ((i != 0) || l.isNil ()) {
      throw new IndexOutOfBoundsException 
	("Attempt to access element " + i + " in list of length " + size () + "\n" + this);
    }

    return l.head ();
  }


  public abstract boolean isNil ();
  public abstract int size ();

  public abstract Object head () throws EmptyListException;
  public abstract List tail () throws EmptyListException;

  public abstract List concat (List l);
  public abstract List reverse ();
  abstract List reverseAux (List end);


  public String toString ()
  {
    StringBuffer sb = new StringBuffer ();

    if (isNil ()) {
      return ("[]");
    }

    sb.append ("[ ");
    List l = this;
    boolean first = true;
    while (!l.isNil ()) {
      if (first) {
	first = false;
      } else {
	sb.append (", ");
      }
      sb.append (l.head ());
      l = l.tail ();
    }
    sb.append (" ]");

    return sb.toString ();
  }


  // Returns a duplicate in the list (upto .equals ()) if there is one, otherwise returns null.
  public Object findDuplicate ()
  {
    HashSet set = new HashSet ();
    List list = this;
    while (!list.isNil ()) {
      Object o = list.head ();
      if (!set.contains (o)) {
	set.add (o);
      } else {
	return o;
      }
      list = list.tail ();
    }
    return null;
  }


  // Applies fun.f to every object in the list to create a new list of the results.
  // If fun.f returns null, the result is discarded (a combined map/filter).
  public List map (Function fun)
  {
    if (isNil ()) {
      return nil ();
    } else {
      Object newHead = fun.f (head ());
      List newTail = tail ().map (fun);
      if (newHead == null) {
	return newTail;
      } else {
	return newTail.cons (newHead);
      }
    }
  }


  public boolean contains (Object o)
  {
    List list = this;
    while (!list.isNil ()) {
      if (o.equals (list.head ())) {
	return true;
      }
      list = list.tail ();
    }
    return false;
  }
}


