import java_cup.runtime.SymbolFactory;
import java_cup.runtime.ComplexSymbolFactory;

%%

%class SkeletonLexer
%public
%{
   private SymbolFactory sf = new ComplexSymbolFactory ();
   public SkeletonLexer (java.io.InputStream r, SymbolFactory sf)
   {
     this (r);
     this.sf = sf;
   }
%}
%eofval{
  return sf.newSymbol ("EOF", sym.EOF);
%eofval}

%unicode

%cup
%cupdebug

%char
%column
%line


ALPHA=[A-Za-z_]
DIGIT=[0-9]
NONNEWLINE_WHITE_SPACE_CHAR=[\ \t\b\012]
NEWLINE=\r|\n|\r\n
IDENT={ALPHA}({ALPHA}|{DIGIT}|_)*

%% 

<YYINITIAL> {
  "(" { return sf.newSymbol ("LeftParen", sym.LPAREN); }
  ")" { return sf.newSymbol ("RightParen", sym.RPAREN); }
  ";" { return sf.newSymbol ("Semicolon", sym.SEMI); }
  "." { return sf.newSymbol ("Equals", sym.DOT); }

  {NONNEWLINE_WHITE_SPACE_CHAR}+ { }

  {IDENT} {
    return sf.newSymbol ("Identifier", sym.IDENTIFIER, yytext ());
  }

  {DIGIT}+ {
    int i = Integer.parseInt (yytext ());
    return sf.newSymbol ("IntegerConstant", sym.INTEGER_CONSTANT, new Integer (i));
  }
}

{NEWLINE} { }

. {
  System.out.println ("Illegal character: <" + yytext () + ">");
}

